import { Body, Controller, Del, Get, Inject, Param, Post, Query } from '@midwayjs/decorator';
import { BaseController } from 'free-midway-component';
import { DataRecordService } from '../../service/record/dataRecord.service';
import { CDTO_dataRecord, CVO_dataRecord, FVO_dataRecord, QDTO_dataRecord, QVO_dataRecord, UDTO_dataRecord, UVAO_dataRecord } from '../../interface/record/dataRecord.interface';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
@ApiTags(['数据操作记录'])
@Controller('/dataRecord')
export class DataRecordController extends BaseController {
  @Inject()
  service: DataRecordService;

  @Post('/')
  @Validate()
  @ApiResponse({ type: CVO_dataRecord })
  async create(@Body() data: CDTO_dataRecord) {
    const dbData = await this.service.create(data);
    const result = new CVO_dataRecord(dbData);
    return result;
  }
  @Get('/')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_dataRecord })
  async query(@Query() filter: QDTO_dataRecord, @Query('skip') skip: number, @Query('limit') limit: number) {
    const list = await this.service.query(filter, { skip, limit });
    const data = [];
    for (const i of list) {
      const newData = new QVO_dataRecord(i);
      data.push(newData);
    }
    const total = await this.service.count(filter);
    return { data, total };
  }

  @Get('/:id')
  @ApiResponse({ type: FVO_dataRecord })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id);
    const result = new FVO_dataRecord(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @ApiResponse({ type: UVAO_dataRecord })
  async update(@Param('id') id: string, @Body() body: UDTO_dataRecord) {
    const result = await this.service.updateOne(id, body);
    return result;
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
