import { JoinPoint, MidwayDecoratorService, REQUEST_OBJ_CTX_KEY, createCustomMethodDecorator } from '@midwayjs/core';
import { FrameworkErrorEnum, ServiceError } from 'free-midway-component';
import { get } from 'lodash';
import { RoleService } from '../service/system/role.service';
export const CHECKPERMISSIONCODE_KEY = 'decorator:check_permission_code';
export const checkPermissionCode = options => {
  return createCustomMethodDecorator(CHECKPERMISSIONCODE_KEY, options);
};
// 第二层及以后的权限,去掉第一层级
const resetRoleMenus = list => {
  list = list.map(i => {
    const arr = i.split('.');
    if (arr.length <= 1) return i;
    arr.shift();
    return arr.join('.');
  });
  return list;
};

export const CheckPermissionCodeInit = (decoratorService: MidwayDecoratorService) => {
  decoratorService.registerMethodHandler(CHECKPERMISSIONCODE_KEY, options => {
    return {
      around: async (joinPoint: JoinPoint) => {
        const roleCode = get(options, 'metadata.roleCode');
        if (roleCode) {
          const instance = joinPoint.target;
          const ctx = instance[REQUEST_OBJ_CTX_KEY];
          const roleService = await ctx.requestContext.getAsync(RoleService);
          const isAdmin = roleService.isSuperAdmin();
          if (!isAdmin) {
            let roleMenus = await roleService.getUserMenus(true);
            roleMenus = resetRoleMenus(roleMenus);
            if (!roleMenus.includes(roleCode)) {
              console.log(options);
              throw new ServiceError('您没有当前功能的权限!', FrameworkErrorEnum.SERVICE_FAULT);
            }
            const result = await joinPoint.proceed(...joinPoint.args);
            return result;
          }
          const result = await joinPoint.proceed(...joinPoint.args);
          return result;
        }
        const result = await joinPoint.proceed(...joinPoint.args);
        return result;
      },
    };
  });
};
