import { JoinPoint, MidwayDecoratorService, REQUEST_OBJ_CTX_KEY, createCustomMethodDecorator } from '@midwayjs/core';
import { MidwayI18nService } from '@midwayjs/i18n';
import { get } from 'lodash';
import { DataRecordService } from '../service/record/dataRecord.service';
export const DATARECORD_KEY = 'decorator:data_record';
export const dataRecord = (options?: any) => {
  return createCustomMethodDecorator(DATARECORD_KEY, options);
};

const getMethodName = (mapping: Map<string, any>, controller: string, path: string) => {
  const map = mapping.get(controller);
  if (!map) return;
  return get(map, path);
};
// const i18n = await ctx.requestContext.getAsync(MidwayI18nService);
// const mapping = i18n.getLocaleMapping(ctx.locale, 'methods');
// const i18nName = getMethodName(mapping, controllerName, methodName);
// console.log(i18nName);

const defaultMethods = ['create', 'update', 'delete'];
export const DataRecordInit = (decoratorService: MidwayDecoratorService) => {
  decoratorService.registerMethodHandler(DATARECORD_KEY, options => {
    return {
      around: async (joinPoint: JoinPoint) => {
        let recordPreData = null;
        let recordService = null;
        try {
          const instance = joinPoint.target;
          const controllerName = options.target.name;
          const methodName = options.propertyName;
          const metaData = options.metadata;
          const ctx = instance[REQUEST_OBJ_CTX_KEY];
          recordService = await ctx.requestContext.getAsync(DataRecordService);
          // 组织记录数据
          recordPreData = await recordService.makeLogs(controllerName, methodName);
          if (get(metaData, 'before')) {
            recordPreData.origin_data = await instance.service[get(metaData, 'before')]();
          } else if (defaultMethods.includes(methodName)) {
            // 如果是create,update,delete 默认方法.那就默认处理.自己写的自己处理

            recordPreData.origin_data = await recordService.getDefaultMethodOriginData(instance.service, methodName);
          }
          // 执行原函数
          const result = await joinPoint.proceed(...joinPoint.args);
          // 如果是create,update,delete 默认方法.那就默认处理.自己写的自己处理
          if (get(metaData, 'after')) {
            recordPreData.new_data = await instance.service[get(metaData, 'after')]();
          } else if (defaultMethods.includes(methodName)) {
            let od = null;
            if (methodName === 'create') od = result;
            else od = recordPreData.origin_data;
            recordPreData.new_data = await recordService.getDefaultMethodNewData(instance.service, methodName, od);
          }
          recordPreData.status = '0';
          return result;
        } catch (error) {
          if (recordPreData) {
            // 有数据的情况下存日志有意义,否则就是只有个status没意义
            recordPreData.status = '1';
          }
          throw error;
        } finally {
          // save logs
          if (recordPreData) await recordService.create(recordPreData);
        }
      },
    };
  });
};
